dnl @synopsis CHECK_GNU_MAKE()
dnl
dnl This macro searches for a GNU version of make.  If a match is found, the
dnl makefile variable `ifGNUmake' is set to the empty string, otherwise it is
dnl set to "#".  This is useful for  including a special features in a Makefile,
dnl which cannot be handled by other versions of make.  The variable
dnl _cv_gnu_make_command is set to the command to invoke GNU make if it exists,
dnl the empty string otherwise.
dnl
dnl Here is an example of its use:
dnl
dnl Makefile.in might contain:
dnl
dnl     # A failsafe way of putting a dependency rule into a makefile
dnl     $(DEPEND):
dnl             $(CC) -MM $(srcdir)/*.c > $(DEPEND)
dnl
dnl     @ifGNUmake@ ifeq ($(DEPEND),$(wildcard $(DEPEND)))
dnl     @ifGNUmake@ include $(DEPEND)
dnl     @ifGNUmake@ endif
dnl
dnl Then configure.in would normally contain:
dnl
dnl     CHECK_GNU_MAKE()
dnl     AC_OUTPUT(Makefile)
dnl
dnl Then perhaps to cause gnu make to override any other make, we could do
dnl something like this (note that GNU make always looks for GNUmakefile first):
dnl
dnl     if  ! test x$_cv_gnu_make_command = x ; then
dnl             mv Makefile GNUmakefile
dnl             echo .DEFAULT: > Makefile ;
dnl             echo \  $_cv_gnu_make_command \$@ >> Makefile;
dnl     fi
dnl
dnl Then, if any (well almost any) other make is called, and GNU make also exists,
dnl then the other make wraps the GNU make.
dnl
dnl @author John Darrington <j.darrington@elvis.murdoch.edu.au>
dnl @version $Id: acinclude.m4,v 1.3 2001/10/28 17:26:48 offby1 Exp $
dnl
AC_DEFUN(
        [CHECK_GNU_MAKE], [ AC_CACHE_CHECK( for GNU make,_cv_gnu_make_command,
                _cv_gnu_make_command='' ;
dnl Search all the common names for GNU make
                for a in "$MAKE" make gmake gnumake ; do
                        if  ( $a --version 2> /dev/null | grep  -q GNU  ) ;  then
                                _cv_gnu_make_command=$a ;
                                break;
                        fi
                done ;
        ) ;
dnl If there was a GNU version, then set @ifGNUmake@ to the empty string, '#' otherwise
        if test  "x$_cv_gnu_make_command" != "x"  ; then
                ifGNUmake='' ;
        else
                ifGNUmake='#' ;
        fi
        AC_SUBST(ifGNUmake)
] )

dnl On some old RedHat systems with egcs-2, when we link, I'd see
dnl
dnl libbridge.a(auction.o): In function `bridge_exception type_info function':
dnl /home/users/erich/local/src/bridge/auction.cpp(.text+0x0): multiple definition of `exception::exception(void)'
dnl main.o(.text+0x0):/home/users/erich/local/src/bridge/main.cpp: first defined here
dnl 
dnl Removing `-fno-default-inline' makes the problem go away, but I
dnl like that option, since I think it makes debugging easier.  So I
dnl only use that option when we're not using egcs-2.
dnl 

AC_DEFUN(
        [EGCS_CHECK], [ AC_CACHE_CHECK( for egcs 2,_cv_egcs,
                _cv_egcs='no' ;
		if  ( "$CXX" 2> /dev/null | egrep -q '^egcs-2\.'  ) ;  then
			_cv_egcs=yes ;
		fi ;
        ) ;
        if test  "x$_cv_egcs" = "xyes"  ; then
                NO_DEFAULT_INLINE='' ;
        else
                NO_DEFAULT_INLINE='-fno-default-inline' ;
        fi
        AC_SUBST(NO_DEFAULT_INLINE)
] )
