// Copyright (C) 2001 Eric Hanchrow
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Eric Hanchrow -- offby1@blarg.net
// 1914 N. 40th
// Seatlle, WA, USA
// 98103-8310
#ifndef FILE_EXCEPTIONS_SEEN
#define FILE_EXCEPTIONS_SEEN

#include <string>
#include <exception>

// all the real code will throw exceptions derived from this.  Test
// code, however, may throw other kinds of exceptions; in particular,
// the CppUnit framework throws exceptions that derive from
// `exception'.  I don't want to catch those by accident.
class bridge_exception : public std::exception {};

#define MAKE_EXCEPTION_CLASS(NAME)              \
class NAME : public bridge_exception            \
{                                               \
public:                                         \
  const char* what () const throw ()            \
  {                                             \
    return #NAME;                               \
  }                                             \
};

MAKE_EXCEPTION_CLASS(Bad_State)
MAKE_EXCEPTION_CLASS(No_Declarer)
MAKE_EXCEPTION_CLASS(No_Bids)
MAKE_EXCEPTION_CLASS(Bad_Multiplier)
MAKE_EXCEPTION_CLASS(Bad_Card)
MAKE_EXCEPTION_CLASS(Bad_Double)
MAKE_EXCEPTION_CLASS(Insufficient_Bid)

class internal_error : public std::exception
{
  std::string _what;
public:
  internal_error (const std::string &what) : _what (what) {}
  ~internal_error () throw () {}
  const char* what () const throw ()
  {
    return _what.c_str();
  }
};

// From Stroustrup, ``The C++ Programming Language'', special edition,
// page 751
template<class X, class A> inline void Assert (A assertion)
{
  if (!assertion) throw X();
}

template<class X, class A> inline void Assert (A assertion, const std::string &what)
{
  if (!assertion) throw X(what);
}

#endif // FILE_EXCEPTIONS_SEEN
