// Copyright (C) 2001 Eric Hanchrow
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Eric Hanchrow -- offby1@blarg.net
// 1914 N. 40th
// Seatlle, WA, USA
// 98103-8310
#ifndef FILE_PLAYER_SEEN
#define FILE_PLAYER_SEEN

#include <vector>
#include "hand.hh"
#include "auction.hh"
#include "card.hh"
#include "trick.hh"
#include "compass.hh"
#include "score.hh"

class player
{
protected:

  hand _hand;

public:

  virtual void pick_up_cards (const hand &h)
  {
    Assert<internal_error>(_hand.size () == 0, "Picked up cards when we still had a hand");
    _hand = h;
  }

  virtual const hand & get_hand () const { return _hand; }

  /// The return value was allocated with `new', so `delete' it when
  /// you're done.
  virtual call *make_call (const seat &who_am_i,
                           const auction &so_far,
                           const rubber_scoresheet &score) = 0;

  /// Decide which card to play to this trick.
  virtual card choose_card (const auction &,
                            const rubber_scoresheet &,
                            const plays &,
                            const std::vector<card> &current_trick,
                            const hand *dummy // will be 0 for opening lead
                            ) = 0;

  virtual ~player () {}
};


#endif  /* FILE_PLAYER_SEEN */
