// Copyright (C) 2001 Eric Hanchrow
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Eric Hanchrow -- offby1@blarg.net
// 1914 N. 40th
// Seatlle, WA, USA
// 98103-8310
#ifndef FILE_SCORE_SEEN
#define FILE_SCORE_SEEN

#include "hand.hh"
#include "call.hh"
#include "trick.hh"
#include "contract.hh"

/// one of the two halves of a scoresheet.  Traditionally labeled
/// either `we' or `they'.
/**
 * \bug -- make this class name visible only to class
 * rubber_scoresheet.
 */
class column
{
  friend class rubber_scoresheet;
  bool _vulnerable;
  unsigned short _above;
  unsigned short _below;
public:
  column ()
    : _vulnerable (false),
      _above (0),
      _below (0)
  {
  }

  // What you and I call `below the line' points, the Laws call `trick
  // points'; they call `above the line' points `premium points'.

  void add_above (unsigned short s);

  /// This may have the side-effect of setting _vulnerable to true.
  void add_below (unsigned short s,
                  bool opponents_vulnerable);

  bool is_vulnerable () const { return _vulnerable; }

  unsigned short above () const { return _above; }
  unsigned short below () const { return _below; }
};

class rubber_scoresheet
{
  column _north_south;
  column _east_west  ;
  bool   _completed;
public:
  rubber_scoresheet () : _completed (false) {}
  void tally_hand (const plays &p,
                   const contract &c);

  void credit_honors (const contract &c,
                      const hand &declarer,
                      const hand &dummy,
                      const hand &lho,
                      const hand &rho);

  /// award bonuses for partscore etc., according to Law 80
  /// (http://www.acbl.org/info/rlaws/part7.stm#law80)
  void abort_rubber ();

  bool rubber_is_over () const { return _completed; }

  bool ns_vulnerable () const { return _north_south.is_vulnerable (); }
  bool ew_vulnerable () const { return _east_west  .is_vulnerable (); }

  const column & north_south () const { return _north_south; }
  const column & east_west   () const { return _east_west  ; }
};

#endif  /* FILE_SCORE_SEEN */
