// Copyright (C) 2001 Eric Hanchrow
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Eric Hanchrow -- offby1@blarg.net
// 1914 N. 40th
// Seatlle, WA, USA
// 98103-8310
#include "../stats.hh"
#include "score-tests.hh"

using namespace CppUnit;

// This is an unspeakably ugly hack.  Shame on me.
// trick::determine_winner modifies this global, which is normally
// defined in main.  I haven't yet thought of a good way to preserve
// the functionality (namely, keeping track of statistics) without
// using a global.
std::map<rank, unsigned int> stats;

namespace {
void
stuff_north_with_tricks (plays &tricks_played)
{
  denomination d;
  card two_of_clubs   (0);
  card three_of_clubs (1);
  std::vector<card> cards;
  cards.push_back (three_of_clubs);
  cards.push_back (two_of_clubs  );
  cards.push_back (two_of_clubs  );
  cards.push_back (two_of_clubs  );
  seat north;
  trick t (north, d, cards);

  tricks_played.push_back (t);
  tricks_played.push_back (t);
  tricks_played.push_back (t);
  tricks_played.push_back (t);
  tricks_played.push_back (t);
  tricks_played.push_back (t);
  tricks_played.push_back (t);
  tricks_played.push_back (t);
  tricks_played.push_back (t);
  tricks_played.push_back (t);
  tricks_played.push_back (t);
  tricks_played.push_back (t);
  tricks_played.push_back (t);
}

void
north_takes_thirteen_tricks (rubber_scoresheet &ss,
                             const contract &c)
{
  plays tricks_played;
  stuff_north_with_tricks (tricks_played);
  ss.tally_hand (tricks_played, c);
}
};

void
ScoreTest::down_seven ()
{
  contract one_notrump_east (bid (1, denomination ()),
                             seat () + 1, // east
                             contract::not_doubled);

  rubber_scoresheet scoresheet;

  north_takes_thirteen_tricks (scoresheet, one_notrump_east);

  CPPUNIT_ASSERT (0 == scoresheet.east_west ().above ());
  CPPUNIT_ASSERT (0 == scoresheet.east_west ().below ());

  // Down 7, not doubled, not vulnerable
  CPPUNIT_ASSERT (350 == scoresheet.north_south ().above ());
  CPPUNIT_ASSERT (000 == scoresheet.north_south ().below ());
}

void
ScoreTest::six_overtricks ()
{
  contract one_notrump_north (bid (1, denomination ()),
                              seat (), // defaults to north
                              contract::not_doubled);

  rubber_scoresheet ss;
  north_takes_thirteen_tricks (ss, one_notrump_north);

  CPPUNIT_ASSERT (40  == ss.north_south ().below ());
  CPPUNIT_ASSERT (180 == ss.north_south ().above ());
}

void
ScoreTest::slam ()
{
  contract six_notrump_doubled_north (bid (6, denomination ()),
                                      seat (), // defaults to north
                                      contract::doubled);

  rubber_scoresheet ss;
  north_takes_thirteen_tricks (ss, six_notrump_doubled_north);

  // Note -- even though we made our contract, there are no
  // below-the-line points.  The trick score *was* below the line, for
  // a millisecond, but then because we won a game, the scorekeeper
  // drew a line below our trick-score points, thus moving them above
  // the line.
  CPPUNIT_ASSERT (0 == ss.north_south ().below ());

  // small slam bonus              : 500
  // double bonus                  :  50
  // one overtrick X2              : 100
  // six NT tricks bid and made, X2: 380

  CPPUNIT_ASSERT (1030 == ss.north_south ().above ());
}

void
ScoreTest::doubled_vulnerable_slam ()
{
  contract three_no_north (bid (3, denomination ()),
                           seat (),
                           contract::not_doubled);

  contract six_no_north (bid (6, denomination ()),
                         seat (), // defaults to north
                         contract::doubled);

  rubber_scoresheet ss;

  north_takes_thirteen_tricks (ss, three_no_north);
  CPPUNIT_ASSERT(  0 == ss.north_south ().below ());
  CPPUNIT_ASSERT(220 == ss.north_south ().above ());

  north_takes_thirteen_tricks (ss, six_no_north);
  CPPUNIT_ASSERT (0 == ss.north_south ().below ());

  // vulnerable small slam        : 750
  // double bonus                 :  50
  // six NT tricks, doubled       : 380
  // doubled, vulnerable overtrick: 200
  // our score from the last game : 220
  // rubber bonus                 : 700
  CPPUNIT_ASSERT (2300 == ss.north_south ().above ());
}

Test *ScoreTest::suite ()
{
  TestSuite *suiteOfTests = new TestSuite;
  suiteOfTests->addTest (new TestCaller<ScoreTest>("down seven"             , &ScoreTest::down_seven));
  suiteOfTests->addTest (new TestCaller<ScoreTest>("six overtricks"         , &ScoreTest::six_overtricks));
  suiteOfTests->addTest (new TestCaller<ScoreTest>("slam"                   , &ScoreTest::slam));
  suiteOfTests->addTest (new TestCaller<ScoreTest>("doubled vulnerable slam", &ScoreTest::doubled_vulnerable_slam));

  return suiteOfTests;
}
