// Copyright (C) 2001 Eric Hanchrow
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Eric Hanchrow -- offby1@blarg.net
// 1914 N. 40th
// Seatlle, WA, USA
// 98103-8310

#if defined (WIN32)
# pragma warning(disable: 4786)
#endif

#include <cassert>
#include "trick.hh"
#include "card.hh"
#include "call.hh"
#include "stats.hh"

std::ostream & operator<<(std::ostream &o, const trick &t)
{
  for (unsigned short cards_examined = 0;
       cards_examined < 4;
       cards_examined++)
    {
      if (cards_examined != 0)
        o << "; ";

      o << t.leader () + cards_examined << ": " << t.cards()[cards_examined];
    }

  o << std::endl;
  return o;
}

void trick::determine_winner (const denomination &trump_suit)
{
  const suit led = _cards [0].get_suit ();
  std::vector<card>::const_iterator highest_of_led_suit = _cards.begin ();
  std::vector<card>::const_iterator highest_trump       = _cards.end   ();

  unsigned short trumps_in_trick = 0;

  for (std::vector<card>::const_iterator i = _cards.begin ();
       i != _cards.end ();
       i++)
    {
      if ((i->get_suit () == led)
          &&
          (i->get_rank () > highest_of_led_suit->get_rank ()))
        {
          highest_of_led_suit = i;
        }

      if (!trump_suit.is_notrump ()
          &&
          (i->get_suit () == trump_suit.get_suit ()))
        {
          trumps_in_trick++;

          if (1 == trumps_in_trick)
            highest_trump = i;
          else if (i->get_rank () > highest_trump->get_rank ())
            {
              highest_trump = i;
            }
        }
    }

  if (highest_trump != _cards.end ())
    {
      _winner = this->leader () + (highest_trump - _cards.begin ());

      std::cout << "Trick was "
                << ((!trump_suit.is_notrump ()
                     &&
                     highest_trump->get_suit ()
                     != highest_of_led_suit->get_suit ())
                    ? "trumped"
                    : "won")
                << " by "
                << _winner
                << " with the "
                << *highest_trump
                << std::endl;

      stats [highest_trump->get_rank ()]++;
    }
  else
    {
      _winner = this->leader () + (highest_of_led_suit - _cards.begin ());

      std::cout << "Trick was won by "
                << _winner
                << " with the "
                << *highest_of_led_suit
                << std::endl;

      stats [highest_of_led_suit->get_rank ()]++;
    }

}
